@@ -813,6 +813,9 @@ FindMarkers.Seurat <- function(
   if (!is.null(x = assay) && !is.null(x = reduction)) {
     stop("Please only specify either assay or reduction.")
   }
+  if (length(x = ident.1) == 0) {
+    stop("At least 1 ident must be specified in `ident.1`")
+  }
   # select which data to use
   if (is.null(x = reduction)) {
     assay <- assay %||% DefaultAssay(object = object)
@@ -836,6 +839,20 @@ FindMarkers.Seurat <- function(
       cells = c(cells$cells.1, cells$cells.2)
     )
   }
+  # check normalization method
+  norm.command <- paste0("NormalizeData.", assay)
+  if (norm.command %in% Command(object = object) && is.null(x = reduction)) {
+    norm.method <- Command(
+      object = object,
+      command = norm.command,
+      value = "normalization.method"
+    )
+    if (norm.method != "LogNormalize") {
+      mean.fxn <- function(x) {
+        return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+      }
+    }
+  }
   de.results <- FindMarkers(
     object = data.use,
     slot = slot,
@@ -856,6 +873,7 @@ FindMarkers.Seurat <- function(
     pseudocount.use = pseudocount.use,
     mean.fxn = mean.fxn,
     base = base,
+    fc.name = fc.name,
     ...
   )
   return(de.results)
@@ -1617,9 +1635,7 @@ MarkerTest <- function(
 # @param data.use Data to test
 # @param cells.1 Group 1 cells
 # @param cells.2 Group 2 cells
-# @param latent.vars Confounding variables to adjust for in DE test. Default is
-# "nUMI", which adjusts for cellular depth (i.e. cellular detection rate). For
-# non-UMI based data, set to nGene instead.
+# @param latent.vars Confounding variables to adjust for in DE test
 # @param verbose print output
 # @param \dots Additional parameters to zero-inflated regression (zlm) function
 # in MAST
@@ -1630,17 +1646,6 @@ MarkerTest <- function(
 # genes.
 #
 #' @importFrom stats relevel
-#
-# @export
-#
-# @examples
-# \dontrun{
-#   data("pbmc_small")
-#   pbmc_small
-#   MASTDETest(pbmc_small, cells.1 = WhichCells(object = pbmc_small, idents = 1),
-#               cells.2 = WhichCells(object = pbmc_small, idents = 2))
-# }
-#
 MASTDETest <- function(
   data.use,
   cells.1,
